﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Combo_BindToEnum
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
         
        public MainWindow()
        {
            InitializeComponent();

            DataContext = new MainVM();
        }
    }
     
    public class MainVM : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        static private MainVM _gInstance = null;
        static public MainVM Instance
        {
            get
            {
                if (_gInstance == null)
                {
                    new MainVM();
                }
                return _gInstance;
            }
            set
            {
                _gInstance = value;
            }
        }

        public MainVM()
        {
            Data = new List<Data>
            {
                new Data { ID = 1, Greeting = Greetings.DaysOfTheWeek, Name = "Mark" },
                new Data { ID = 2, Greeting = Greetings.Christmas, Name = "Jenny" },
                new Data { ID = 3, Greeting = Greetings.Birthday, Name = "James" },
            };

            GreetingsCollection = new GreetingsCollection();

            if (_gInstance == null)
            {
                _gInstance = this;
            }
        }

        public List<Data> Data { get; set; }
        private DaysOfTheWeek _Day = DaysOfTheWeek.Monday;
        public DaysOfTheWeek Day
        {
            get
            {
                return _Day;
            }
            set
            {
                _Day = value;
                OnPropertyChanged("Day");
                Debug.WriteLine("Day selected: " + value.ToString());

                GreetingsCollection = new GreetingsCollection();
            }
        }

        private GreetingsCollection _GreetingsCollection;
        public GreetingsCollection GreetingsCollection
        {
            get { return _GreetingsCollection; }
            set
            {
                _GreetingsCollection = value;
                OnPropertyChanged("GreetingsCollection");
            }
        }
    }

    public class Data : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        public int ID { get; set; }
        public string Name { get; set; }

        private Greetings _Greeting;
        public Greetings Greeting
        {
            get { return _Greeting; }
            set
            {
                _Greeting = value;
                OnPropertyChanged("Greeting");
            }
        }
    }

    public class DaysOfTheWeekCollection : EnumDescrioptionWrapperCollection<DaysOfTheWeek> { };
    public class GreetingsCollection : EnumDescrioptionWrapperCollection<Greetings> { };

    public class EnumDescrioptionWrapperCollection<T> : List<EnumDescriptionkWrapper<T>> where T : struct, IConvertible
    {
        public EnumDescrioptionWrapperCollection()
        {

            foreach (var dow in Enum.GetValues(typeof(T)))
            {
                EnumDescriptionkWrapper<T> t = new EnumDescriptionkWrapper<T>((T)dow);
                Add(t);
            }    
        }
    }

    public class EnumDescriptionkWrapper<T> where T : struct, IConvertible
    {
        T _enumVal;
        public EnumDescriptionkWrapper(T enumVal)
        {
            if (!typeof(T).IsEnum)
            {
                throw new ArgumentException("T must be an enumerated type");
            }

            _enumVal = enumVal;
        }

        public T Value 
        { 
            get { return _enumVal; } 
        }

        public string Description
        {
            get
            {
                DescriptionAttribute description = _enumVal.GetType().GetMember(_enumVal.ToString())[0].GetCustomAttributes(false).FirstOrDefault() as DescriptionAttribute;
                if (description.Description == "Happy DayOfTheWeek")
                {

                    string desc = GetDescription(MainVM.Instance.Day);
                    return desc;

                }
                else
                {
                    return description.Description;
                }

            }
        }
        private string GetDescription(DaysOfTheWeek enumVal)
        {
            DescriptionAttribute description = enumVal.GetType().GetMember(enumVal.ToString())[0].GetCustomAttributes(false).FirstOrDefault() as DescriptionAttribute;
            return description.Description;
        }
    }

    public enum DaysOfTheWeek
    {
        [Description("Happy Monday")]
        Monday,
        [Description("Happy Tuesday")]
        Tuesday,
        [Description("Happy Wednesday")]
        Wednesday,
        [Description("Good Friday")]
        Friday,
        [Description("Cool Sunday")]
        Sunday

    }

    public enum Greetings
    {
        [Description("Happy Birthday")]
        Birthday,
        [Description("Merry Christmas")]
        Christmas,
        [Description("Happy DayOfTheWeek")]
        DaysOfTheWeek,
        [Description("Happy New Year")]
        NewYear,
        [Description("Happy Thanksgiving")]
        Thanksgiving

    }
   
}
